<?php

namespace App\Entity;

use App\Repository\OrganizationalUnitRepository;
use Gedmo\Mapping\Annotation as Gedmo;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

#[Gedmo\Tree(type: 'materialized-path')]
#[ORM\Entity(repositoryClass: OrganizationalUnitRepository::class)]
class OrganizationalUnit extends AbstractEntity
{
    use NameableTrait;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $description = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $comments = null;

    #[Gedmo\TreePath(separator: '/', appendId: true)]
    #[ORM\Column(length: 255, nullable: true)]
    private ?string $path = null;

    #[Gedmo\TreeParent]
    #[ORM\ManyToOne(targetEntity: self::class, inversedBy: 'organizationalUnits')]
    private ?self $parent = null;

    /**
     * @var Collection<int, self>
     */
    #[ORM\OneToMany(targetEntity: self::class, mappedBy: 'parent')]
    private Collection $organizationalUnits;

    #[ORM\ManyToOne(inversedBy: 'organizationalUnits')]
    private ?NetworkSettings $networkSettings = null;

    /**
     * @var Collection<int, User>
     */
    #[ORM\ManyToMany(targetEntity: User::class, mappedBy: 'allowedOrganizationalUnits')]
    private Collection $users;

    public function __construct()
    {
        parent::__construct();
        $this->organizationalUnits = new ArrayCollection();
        $this->users = new ArrayCollection();
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): static
    {
        $this->description = $description;

        return $this;
    }

    public function getComments(): ?string
    {
        return $this->comments;
    }

    public function setComments(?string $comments): static
    {
        $this->comments = $comments;

        return $this;
    }

    public function getPath(): ?string
    {
        return $this->path;
    }

    public function setPath(?string $path): static
    {
        $this->path = $path;

        return $this;
    }

    public function getParent(): ?self
    {
        return $this->parent;
    }

    public function setParent(?self $parent): static
    {
        $this->parent = $parent;

        return $this;
    }

    /**
     * @return Collection<int, self>
     */
    public function getOrganizationalUnits(): Collection
    {
        return $this->organizationalUnits;
    }

    public function addOrganizationalUnit(self $organizationalUnit): static
    {
        if (!$this->organizationalUnits->contains($organizationalUnit)) {
            $this->organizationalUnits->add($organizationalUnit);
            $organizationalUnit->setParent($this);
        }

        return $this;
    }

    public function removeOrganizationalUnit(self $organizationalUnit): static
    {
        if ($this->organizationalUnits->removeElement($organizationalUnit)) {
            // set the owning side to null (unless already changed)
            if ($organizationalUnit->getParent() === $this) {
                $organizationalUnit->setParent(null);
            }
        }

        return $this;
    }

    public function getNetworkSettings(): ?NetworkSettings
    {
        return $this->networkSettings;
    }

    public function setNetworkSettings(?NetworkSettings $networkSettings): static
    {
        $this->networkSettings = $networkSettings;

        return $this;
    }

    /**
     * @return Collection<int, User>
     */
    public function getUsers(): Collection
    {
        return $this->users;
    }

    public function addUser(User $user): static
    {
        if (!$this->users->contains($user)) {
            $this->users->add($user);
            $user->addAllowedOrganizationalUnit($this);
        }

        return $this;
    }

    public function removeUser(User $user): static
    {
        if ($this->users->removeElement($user)) {
            $user->removeAllowedOrganizationalUnit($this);
        }

        return $this;
    }
}
