<?php

namespace App\State\Processor;

use ApiPlatform\Metadata\Delete;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Post;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\ProcessorInterface;
use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\UserInput;
use App\Dto\Output\UserOutput;
use App\Repository\UserRepository;

class UserProcessor implements ProcessorInterface
{
    public function __construct(
        private UserRepository $userRepository,
        private ValidatorInterface $validator
    )
    {
    }

    public function process(mixed $data, Operation $operation, array $uriVariables = [], array $context = [])
    {
        switch ($operation){
            case $operation instanceof Post:
            case $operation instanceof Put:
            case $operation instanceof Patch:
                return $this->processCreateOrUpdate($data, $operation, $uriVariables, $context);
           case $operation instanceof Delete:
                return $this->processDelete($data, $operation, $uriVariables, $context);
        }
    }

    private function processCreateOrUpdate($data, Operation $operation, array $uriVariables = [], array $context = []): UserOutput
    {
        if (!($data instanceof UserInput)) {
            throw new \Exception(sprintf('data is not instance of %s', UserInput::class));
        }

        $entity = null;
        if (isset($uriVariables['uuid'])) {
            $entity = $this->userRepository->findOneByUuid($uriVariables['id']);
        }

        $user = $data->createOrUpdateEntity($entity);
        $this->validator->validate($user);
        $this->userRepository->save($user);

        return new UserOutput($user);
    }

    private function processDelete($data, Operation $operation, array $uriVariables = [], array $context = []): null
    {
        $user = $this->userRepository->findOneByUuid($uriVariables['id']);
        $this->userRepository->delete($user);

        return null;
    }
}
